
#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <sys/types.h>
#include <signal.h>
#include <sched.h>
#include <stdlib.h>
#include <unistd.h>

#ifndef SUB_CALLING
# include <stdio.h>
# include <gtk/gtk.h>

# include "interface.h"
# include "support.h"

# include "callbacks.h"
# include "pfunc.h"
# include "sfunc.h"
# include "mfunc.h"
# include "telefono.h"
# include "modem.h"
#else
# define SUB_CALLING
# include "modem.h"
# include "modem.c"
#endif


int SyncCall;

void
on_btok_clicked (GtkButton * button, gpointer user_data)
{
  GtkWidget *entry;
  char text[50], *tnum;
  int h_modem;

  if (Status[S_TELEFONO] == 2)
    return;

  if (Status[S_MTERM])
  {
   /* Il toggle del terminale viene disabilitato 
    * durante luso del terminale 
    */
   Status[S_MTERM] = -1;
   gtk_widget_show (Terminale);
  }  

  entry = lookup_widget (GTK_WIDGET (Telefono), "numero");
  tnum = g_strdup_printf ("%s", gtk_entry_get_text (GTK_ENTRY (entry)));

  if (tnum[0])
    {
      Status[S_TELEFONO] = 2;

      g_print ("\nNumero di telefono: %s", tnum);
      h_modem = open_modem ("/dev/modem", "/var/lock/LCK..modem");
      Status[S_MODEM] = h_modem;

      if (h_modem > -1)
	{
	  if (Status[S_MTERM])
	    SyncCall =
	      gdk_input_add (h_modem, GDK_INPUT_READ, sync_term, user_data);

	  /* reset del modem e del delayed time */
	  send_modem (h_modem, MODEM_INIT_STRING);
	  sleep (2);

	  /* chiamata */
	  sprintf (text, "%s%s\r", MODEM_DIAL_TEL, tnum);
	  send_modem (h_modem, text);
	}
      else
	Status[S_TELEFONO] = 1;	/* serve per non eliminare il lock che non e` stato mai creato */
    }
}


void
on_btno_clicked (GtkButton * button, gpointer user_data)
{
  if (Status[S_MODEM] > 0)
    {

      g_print ("\nScollego il telefono (%d)", Status[S_MODEM]);

      /* Bisogna scollegare la funzione di sync pena la saturazione della CPU */
      if (Status[S_MTERM])
	gdk_input_remove (SyncCall);

      /* g_print("\nClosing the line... now"); */

      /* Chiusura del modem */
      send_modem (Status[S_MODEM], MODEM_CLOSE_STRING);
      close (Status[S_MODEM]);
      Status[S_MODEM] = -1;
    }

  if (!access ("/var/lock/LCK..modem", F_OK) && Status[S_TELEFONO] == 2)
    if (unlink ("/var/lock/LCK..modem"))
      show_error_msg
	("ERRORE: non riesco a liberare il modem cancellando il file /var/lock/LCK..modem");
  Status[S_TELEFONO] = 1;
}


void
on_telefono_destroy (GtkObject * object, gpointer user_data)
{
  g_print ("\nTelefono distrutto");

  on_btno_clicked (NULL, NULL);

  /* Non serve piu` 
     if (!access ("/var/lock/LCK..modem", F_OK) && Status[S_TELEFONO] == 2)
     if (unlink ("/var/lock/LCK..modem"))
     show_error_msg
     ("ERRORE: non riesco a liberare il modem cancellando il file /var/lock/LCK..modem");
   */

  Status[S_TELEFONO] = 0;
  gtk_widget_hide (Terminale);
}


void
on_bt1_clicked (GtkButton * button, gpointer user_data)
{
  inserisci_digit (1);
}


void
on_bt2_clicked (GtkButton * button, gpointer user_data)
{
  inserisci_digit (2);
}


void
on_bt3_clicked (GtkButton * button, gpointer user_data)
{
  inserisci_digit (3);
}


void
on_bt4_clicked (GtkButton * button, gpointer user_data)
{
  inserisci_digit (4);
}


void
on_bt5_clicked (GtkButton * button, gpointer user_data)
{
  inserisci_digit (5);
}


void
on_bt6_clicked (GtkButton * button, gpointer user_data)
{
  inserisci_digit (6);
}


void
on_bt7_clicked (GtkButton * button, gpointer user_data)
{
  inserisci_digit (7);
}


void
on_bt8_clicked (GtkButton * button, gpointer user_data)
{
  inserisci_digit (8);
}


void
on_bt9_clicked (GtkButton * button, gpointer user_data)
{
  inserisci_digit (9);
}


void
on_bt0_clicked (GtkButton * button, gpointer user_data)
{
  inserisci_digit (0);
}


void
on_terminal_toggled (GtkToggleButton * togglebutton, gpointer user_data)
{
  static int richiamo = 0;

  if (Status[S_MTERM] != 0 && Status[S_MTERM] != 1)
    {
      /* Mentre il terminale funziona questa variabile 
       * non deve essere modificata il richiamo serve 
       * per evitare di cadere in loop infinito 
       */
      gtk_toggle_button_set_state (togglebutton, 1);
      g_print ("\nERROR: button toggled not permited during use of terminal");
      richiamo = 1;
    }
  else
    {
      g_print ("\nButton toggled from %d to %d", Status[S_MTERM],
	       togglebutton->active);
      if (GTK_TOGGLE_BUTTON (togglebutton)->active)
	Status[S_MTERM] = 1;
      else
	Status[S_MTERM] = 0;
    }
}


void
inserisci_digit (int digit)
{
  GtkWidget *entry;
  char text[50];

  entry = lookup_widget (GTK_WIDGET (Telefono), "numero");
  sprintf (text, "%s%d", gtk_entry_get_text (GTK_ENTRY (entry)), digit);
  gtk_entry_set_text (GTK_ENTRY (entry), text);
}



/************** SEZIONE TERMINALE *******************/

void
on_terminale_destroy (GtkObject * object, gpointer user_data)
{
  g_print ("\nChiusura del terminale");
  Status[S_MTERM] = (Status[S_MTERM]) ? 1 : 0;

  Terminale = create_terminale ();
}


void
on_termclose_clicked (GtkButton * button, gpointer user_data)
{
  /* in modo da fermare il ciclo */
  Status[S_MTERM] = (Status[S_MTERM]) ? 1 : 0;

  gtk_widget_hide (Terminale);
}

void
sync_term (gpointer data, gint source, GdkInputCondition condition)
{
  GtkWidget *text;
  static int nvolte = 0;
  char buffer[255];		/* Input buffer */
  char *bufptr;			/* Current char in buffer */
  int nbytes;			/* Number of bytes read */

  nvolte++;
  g_print ("\nSync Term started: %d\n", nvolte);

  text = lookup_widget (Terminale, "termtext");

  /* read characters into our string buffer until we get a CR or NL */
  bufptr = buffer;
  while (
	 (nbytes =
	  read (Status[S_MODEM], bufptr,
		buffer + sizeof (buffer) - bufptr - 1)) > 0)
    {
      bufptr += nbytes;
      if (bufptr[-1] == '\n' || bufptr[-1] == '\r')
	break;
    }

  /* nul terminate the string and see if we got an OK response */
  *bufptr = '\0';

  /* se la stringa contiene qualcosa la stampa
   * deve fare anche un controllo sull'esistenza del terminale 
   */
  if (buffer[0] && Status[S_MTERM])
    {
      g_print ("%s", buffer);
      gtk_text_insert (GTK_TEXT (text), FixedFont, NULL, NULL, buffer, -1);
    }

  return;
}
